/*  gtkclock.c -	Analog clock display with 1/10-sec resolution
 *
 *  Matthew J. Gelhaus
 *  October 26, 1998
 */
 
#include <math.h>
#include <stdio.h>
#include <gtk/gtk.h>

#include "gtkclock.h"

#define CLOCK_DEFAULT_SIZE 100

/*
 *  Forward declararations
 */
static void gtk_clock_class_init               (GtkClockClass *klass);
static void gtk_clock_init                     (GtkClock *clock);
static void gtk_clock_destroy                  (GtkObject *object);
static void gtk_clock_realize                  (GtkWidget *widget);
static void gtk_clock_size_request             (GtkWidget *widget,
					                    GtkRequisition *requisition);
static void gtk_clock_size_allocate            (GtkWidget *widget,
					                    GtkAllocation *allocation);
static gint gtk_clock_expose                   (GtkWidget *widget,
						                GdkEventExpose *event);

/* Local data */

static GtkWidgetClass *parent_class = NULL;

guint gtk_clock_get_type ()
{
  static guint clock_type = 0;

  if (!clock_type)
    {
      GtkTypeInfo clock_info =
      {
	"GtkClock",
	sizeof (GtkClock),
	sizeof (GtkClockClass),
	(GtkClassInitFunc)gtk_clock_class_init,
	(GtkObjectInitFunc)gtk_clock_init,
	(GtkArgSetFunc)NULL,
	(GtkArgGetFunc)NULL,
      };

      clock_type = gtk_type_unique(gtk_widget_get_type (), &clock_info);
    }

  return clock_type;
}

static void gtk_clock_class_init (GtkClockClass *class)
{
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;

  object_class = (GtkObjectClass*) class;
  widget_class = (GtkWidgetClass*) class;

  parent_class = gtk_type_class(gtk_widget_get_type ());

  object_class->destroy = gtk_clock_destroy;

  widget_class->realize = gtk_clock_realize;
  widget_class->expose_event = gtk_clock_expose;
  widget_class->size_request = gtk_clock_size_request;
  widget_class->size_allocate = gtk_clock_size_allocate;
}

static void gtk_clock_init (GtkClock *clock)
{
  GTK_WIDGET_SET_FLAGS(GTK_WIDGET(clock), GTK_BASIC);

  clock->uiCurrentTime = 0;
  clock->radius = 0;
  clock->pointer_width = 0;
  clock->fSecAngle = 0.0;
  clock->fMinAngle = 0.0;
  clock->fHourAngle = 0.0;
}

GtkWidget* gtk_clock_new ()
{
  GtkClock *clock;

  clock = gtk_type_new(gtk_clock_get_type ());

  return GTK_WIDGET(clock);
}

static void gtk_clock_destroy (GtkObject *object)
{
  GtkClock *clock;

  g_return_if_fail(object != NULL);
  g_return_if_fail(GTK_IS_CLOCK(object));

  clock = GTK_CLOCK(object);

  if (GTK_OBJECT_CLASS(parent_class)->destroy)
    (* GTK_OBJECT_CLASS(parent_class)->destroy) (object);
}


static void gtk_clock_realize (GtkWidget *widget)
{
  GtkClock *clock;
  GdkWindowAttr attributes;
  gint attributes_mask;

  g_return_if_fail(widget != NULL);
  g_return_if_fail(GTK_IS_CLOCK(widget));

  GTK_WIDGET_SET_FLAGS (widget, GTK_REALIZED);
  clock = GTK_CLOCK(widget);

  attributes.x = widget->allocation.x;
  attributes.y = widget->allocation.y;
  attributes.width = widget->allocation.width;
  attributes.height = widget->allocation.height;
  attributes.wclass = GDK_INPUT_OUTPUT;
  attributes.window_type = GDK_WINDOW_CHILD;
  attributes.event_mask = gtk_widget_get_events(widget) | 
                          GDK_EXPOSURE_MASK;
  attributes.visual = gtk_widget_get_visual(widget);
  attributes.colormap = gtk_widget_get_colormap(widget);

  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
  widget->window = gdk_window_new(widget->parent->window, &attributes,
                                  attributes_mask);

  widget->style = gtk_style_attach(widget->style, widget->window);

  gdk_window_set_user_data(widget->window, widget);

  gtk_style_set_background(widget->style, widget->window, GTK_STATE_ACTIVE);
}

static void gtk_clock_size_request (GtkWidget *widget,
		                    GtkRequisition *requisition)
{
  requisition->width = CLOCK_DEFAULT_SIZE;
  requisition->height = CLOCK_DEFAULT_SIZE;
}

static void gtk_clock_size_allocate (GtkWidget *widget,
			                GtkAllocation *allocation)
{
  GtkClock *clock;

  g_return_if_fail(widget != NULL);
  g_return_if_fail(GTK_IS_CLOCK(widget));
  g_return_if_fail(allocation != NULL);

  widget->allocation = *allocation;
  clock = GTK_CLOCK(widget);

  if (GTK_WIDGET_REALIZED(widget))
    {

      gdk_window_move_resize (widget->window,
			      allocation->x, allocation->y,
			      allocation->width, allocation->height);

    }
  clock->radius = MIN(allocation->width,allocation->height) * 0.45;
  clock->pointer_width = clock->radius / 5;
}

static gint gtk_clock_expose (GtkWidget *widget, GdkEventExpose *event)
{
  GtkClock *clock;
  GdkPoint points[3];
  gdouble s,c;
  gdouble theta;
  gint xc, yc;
  gint tick_length;
  gint i;

  g_return_val_if_fail(widget != NULL, FALSE);
  g_return_val_if_fail(GTK_IS_CLOCK(widget), FALSE);
  g_return_val_if_fail(event != NULL, FALSE);

  if (event->count > 0)
    return FALSE;
  
  clock = GTK_CLOCK(widget);

  gdk_window_clear_area (widget->window,
			 0, 0,
			 widget->allocation.width,
			 widget->allocation.height);

  xc = widget->allocation.width/2;
  yc = widget->allocation.height/2;

  /* Draw ticks */

  for (i=0; i<60; i++)
    {
      theta = (i*M_PI/30.0 -M_PI/2.0);
      s = sin(theta);
      c = cos(theta);

      tick_length = (i%5 == 0) ? clock->pointer_width : clock->pointer_width/3;
      
      gdk_draw_line (widget->window,
		     widget->style->fg_gc[widget->state],
		     xc + c*(clock->radius -tick_length),
		     yc - s*(clock->radius -tick_length),
		     xc + c*clock->radius,
		     yc - s*clock->radius);
    }

  /* Draw pointers */

  s = sin(clock->fHourAngle);
  c = cos(clock->fHourAngle);

  points[0].x = xc + s*clock->pointer_width/2;
  points[0].y = yc + c*clock->pointer_width/2;
  points[1].x = xc + c*clock->radius/2;
  points[1].y = yc - s*clock->radius/2;
  points[2].x = xc - s*clock->pointer_width/2;
  points[2].y = yc - c*clock->pointer_width/2;

  gtk_draw_polygon (widget->style,
		    widget->window,
		    GTK_STATE_NORMAL,
		    GTK_SHADOW_OUT,
		    points, 3,
		    TRUE);
  
  s = sin(clock->fMinAngle);
  c = cos(clock->fMinAngle);

  points[0].x = xc + s*clock->pointer_width/2;
  points[0].y = yc + c*clock->pointer_width/2;
  points[1].x = xc + c*clock->radius*2/3;
  points[1].y = yc - s*clock->radius*2/3;
  points[2].x = xc - s*clock->pointer_width/2;
  points[2].y = yc - c*clock->pointer_width/2;

  gtk_draw_polygon (widget->style,
		    widget->window,
		    GTK_STATE_NORMAL,
		    GTK_SHADOW_OUT,
		    points, 3,
		    TRUE);
  
  s = sin(clock->fSecAngle);
  c = cos(clock->fSecAngle);

  points[0].x = xc + s*clock->pointer_width/4;
  points[0].y = yc + c*clock->pointer_width/4;
  points[1].x = xc + c*clock->radius;
  points[1].y = yc - s*clock->radius;
  points[2].x = xc - s*clock->pointer_width/4;
  points[2].y = yc - c*clock->pointer_width/4;

  gtk_draw_polygon (widget->style,
		    widget->window,
		    GTK_STATE_NORMAL,
		    GTK_SHADOW_OUT,
		    points, 3,
		    TRUE);
  
  return FALSE;
}


void gtk_clock_set_time(GtkClock *clock, guint16 uiNewTime)
{
  unsigned int uiHour, uiMin, uiSec;
//  unsigned int uiTenths;
  float fSec;

  g_return_if_fail(clock != NULL);
  g_return_if_fail(GTK_IS_CLOCK(clock));

  /*
   *  Store the new time
   */
  clock->uiCurrentTime = uiNewTime;

  /*
   *  Compute the new angles
   */
  uiHour   = uiNewTime/36000;
  uiMin    = (uiNewTime -uiHour*36000)/600;
  uiSec    = (uiNewTime -uiHour*36000 -uiMin*600)/10;
//  uiTenths = uiNewTime % 10;
  
//  fSec = (float)uiSec + (float)uiTenths/10;
  fSec = (float)uiSec;
  clock->fSecAngle = -fSec/60.0*2*M_PI +M_PI/2.0;

  clock->fMinAngle = -(float)uiMin/60.0*2*M_PI +M_PI/2.0;
  
  clock->fHourAngle = -(float)uiHour/12.0*2*M_PI +M_PI/2.0;
  
  gtk_widget_draw(GTK_WIDGET(clock), NULL);
}
