/*  gtkled.c -	Implements an LED-style widget.  This widget presents
 *		an indicator that can be set to any color.
 *
 *  Matthew J. Gelhaus
 *  October 23, 1998
 */
 
#include "gtkled.h"

/* Forward declarations */
static void gtk_led_class_init(GtkLEDClass *class);
static void gtk_led_init(GtkLED *LED);
static void gtk_led_destroy(GtkObject *object);
static void gtk_led_realize(GtkWidget *widget);
static void gtk_led_size_request(GtkWidget *widget, 
                                 GtkRequisition *requisition);
static void gtk_led_size_allocate(GtkWidget *widget,
                                  GtkAllocation *allocation);
static gint gtk_led_expose(GtkWidget *widget, GdkEventExpose *event);


/* Local data */
static GtkWidgetClass *parent_class = NULL;


guint gtk_led_get_type()
{
  static guint uiLEDtype = 0;
  
  if (!uiLEDtype)
  {
    GtkTypeInfo LED_info =
    {
      "GtkLED",
      sizeof(GtkLED),
      sizeof(GtkLEDClass),
      (GtkClassInitFunc) gtk_led_class_init,
      (GtkObjectInitFunc) gtk_led_init,
      (GtkArgSetFunc) NULL,
      (GtkArgSetFunc) NULL,
    };
    
    uiLEDtype = gtk_type_unique(gtk_widget_get_type(), &LED_info);
  }
  
  return uiLEDtype;
}


static void gtk_led_class_init(GtkLEDClass *class)
{
  GtkObjectClass *object_class;
  GtkWidgetClass *widget_class;
  
  object_class    = (GtkObjectClass*) class;
  widget_class    = (GtkWidgetClass*) class;

  parent_class = gtk_type_class(gtk_widget_get_type());
  
  object_class->destroy = gtk_led_destroy;
  
  widget_class->realize = gtk_led_realize;
  widget_class->expose_event = gtk_led_expose;
  widget_class->size_request = gtk_led_size_request;
  widget_class->size_allocate = gtk_led_size_allocate;
}


static void gtk_led_init(GtkLED *LED)
{
  /*
   *  gtkLED is a 'basic' widget, i.e. it handles no user input
   */
  GTK_WIDGET_SET_FLAGS(GTK_WIDGET(LED), GTK_BASIC);

  LED->uiRed  = (guint16)0;
  LED->uiGreen = (guint16)0;
  LED->uiBlue  = (guint16)0;
}


GtkWidget* gtk_led_new()
{
  GtkWidget* LED;
  
  LED = gtk_type_new(gtk_led_get_type());
  
  return GTK_WIDGET(LED);
}


static void gtk_led_destroy(GtkObject *object)
{
  GtkLED *LED;
  
  g_return_if_fail(object != NULL);
  g_return_if_fail(GTK_IS_LED(object));
  
  LED = GTK_LED(object);
  
  if (GTK_OBJECT_CLASS(parent_class)->destroy)
    (* GTK_OBJECT_CLASS(parent_class)->destroy)(object);
}


void gtk_led_get_color(GtkLED *LED, guint16 *red, guint16 *green, 
                       guint16 *blue)
{
  g_return_if_fail(LED != NULL);
  g_return_if_fail(GTK_IS_LED(LED));
  
  *red   = LED->uiRed;
  *green = LED->uiGreen;
  *blue  = LED->uiBlue;
}


void gtk_led_set_color(GtkLED *LED, guint16 red, guint16 green, guint16 blue)
{
  g_return_if_fail(LED != NULL);
  g_return_if_fail(GTK_IS_LED(LED));
  
  LED->uiRed   = red;
  LED->uiGreen = green;
  LED->uiBlue  = blue;
  
  if GTK_WIDGET_REALIZED(GTK_WIDGET(LED))
    gtk_widget_draw (GTK_WIDGET(LED), NULL);
}


static void gtk_led_realize(GtkWidget *widget)
{
  GtkLED	*LED;
  GdkWindowAttr	attributes;
  gint		attributes_mask;
  
  g_return_if_fail(widget != NULL);
  g_return_if_fail(GTK_IS_LED(widget));
  
  GTK_WIDGET_SET_FLAGS(widget, GTK_REALIZED);
  LED = GTK_LED(widget);
  
  attributes.x = widget->allocation.x;
  attributes.y = widget->allocation.y;
  attributes.width = widget->allocation.width;
  attributes.height = widget->allocation.height;
  attributes.wclass = GDK_INPUT_OUTPUT;
  attributes.window_type = GDK_WINDOW_CHILD;
  attributes.event_mask = gtk_widget_get_events(widget) | GDK_EXPOSURE_MASK;
  attributes.visual = gtk_widget_get_visual(widget);
  attributes.colormap = gtk_widget_get_colormap(widget);
  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
  
  widget->window = gdk_window_new(widget->parent->window,
                                  &attributes, attributes_mask);
  widget->style = gtk_style_attach(widget->style, widget->window);
  
  gdk_window_set_user_data(widget->window, widget);
  
  gtk_style_set_background(widget->style, widget->window, GTK_STATE_ACTIVE);
}


static void gtk_led_size_request(GtkWidget *widget, GtkRequisition *requisition)
{
  requisition->width = LED_DEFAULT_WIDTH;
  requisition->height = LED_DEFAULT_HEIGHT;
}


static void gtk_led_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
{
  GtkLED *LED;
  
  g_return_if_fail(widget != NULL);
  g_return_if_fail(GTK_IS_LED(widget));
  g_return_if_fail(allocation != NULL);
  
  widget->allocation = *allocation;
  if (GTK_WIDGET_REALIZED(widget))
  {
    LED = GTK_LED(widget);
    
    gdk_window_move_resize(widget->window,
                           allocation->x, allocation->y,
                           allocation->width, allocation->height);
  }
}


static gint gtk_led_expose(GtkWidget *widget, GdkEventExpose *event)
{
  GtkLED	*LED;
  GdkGCValues 	values;
  GdkColormap	*colormap;
  GdkColor	color;
  
  g_return_val_if_fail(widget != NULL, FALSE);
  g_return_val_if_fail(GTK_IS_LED(widget), FALSE);
  
  if (event->count > 0)
    return FALSE;
    
  LED = GTK_LED(widget);
  
  color.red   = LED->uiRed;
  color.green = LED->uiGreen;
  color.blue  = LED->uiBlue;
  colormap=gtk_widget_get_colormap(widget);
  gdk_color_alloc(colormap, &color);

  if (!LED->gc) 
  {
    values.foreground=color;
    values.function=GDK_COPY;
    LED->gc = gdk_gc_new_with_values(widget->window,
                                     &values,
                                     GDK_GC_FUNCTION | GDK_GC_FOREGROUND);
  }
  
  g_return_val_if_fail(event != NULL, FALSE);

  gdk_gc_set_foreground(LED->gc, &color);
  gdk_draw_rectangle(widget->window,
                     LED->gc,
                     TRUE,
                     0, 0,
                     widget->allocation.width, widget->allocation.height);
//  gdk_draw_pixmap(widget->window,
//                  LED->gc,
//                  LED->pixmap,
//                  event->area.x, event->area.y,
//                  event->area.x, event->area.y,
//    event->area.width, event->area.height);
//  gdk_window_clear_area(widget->window, 0, 0, 
//                        widget->allocation.width, widget->allocation.height);

  /* Allocate and set the color */
//  gdk_color_alloc(gtk_widget_get_colormap(widget), &(LED->color));
  
  /* Draw the rectangle */
//  gdk_draw_rectangle(widget->window, widget->style->fg_gc[widget->state],
//  gdk_draw_rectangle(widget->window, widget->style->black_gc,
//                     TRUE, 0, 0, 
//                     widget->allocation.width, widget->allocation.height);
  
  return FALSE;
}


  