/*  main.c -	Main source file for Stones of Asterind game.
 *
 *  Stones of Asterind
 *
 *  Matthew J. Gelhaus
 *  October 20, 1998
 */
 
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <gtk/gtk.h>

#include "main.h"
#include "globals.h"
#include "menu.h"
#include "gameboard.h"
#include "dialogs.h"
#include "newgame.h"
#include "hiscores.h"
#include "gtkled.h"
#include "gtkclock.h"


/*
 *  Save game options
 */
void SaveGameOptions()
{
  FILE	*OutFile;
  
  OutFile = fopen(".sa_options", "w");
  if (OutFile)
  {
    fprintf(OutFile, "%d,%d,%d,%d", iNumColors, iNumRows, iNumCols,
            bAllowRepeatColors);
    fclose(OutFile);
  }
}


/*
 *  Load game options
 */
void LoadGameOptions()
{
  FILE	*InFile;
  
  InFile = fopen(".sa_options", "r");
  if (InFile)
  {
    fscanf(InFile, "%d,%d,%d,%d", &iNumColors, &iNumRows, &iNumCols,
           &bAllowRepeatColors);
    fclose(InFile);
  }
}


/*
 *  Guess button callback - make a guess and advance to the next row
 */
void Guess(GtkWidget *wButton, gpointer pData)
{
  int *iCurrentRow;
  int iCol1, iCol2;
  int iBlacks, iWhites;
  int iPegAccounted[COLS], iSolAccounted[COLS];
  int iNewScore, iChanges;
  char sScore[12];
  
  /*
   *  Get the current row
   */
  iCurrentRow = (int *)pData;
  
  /*
   *  Check for valid row, if not, then game hasn't started yet
   *  (kludgy workaround)
   */
  if (*iCurrentRow == 0)
  {
    *iCurrentRow += 1;
    return;
  }
  
  /*
   *  Disable the current row and display the hint pegs
   */
  if (*iCurrentRow <= iNumRows)
  {
    DisableRow(*iCurrentRow);
    iBlacks = iWhites = 0;
    for (iCol1 = 0; iCol1 < iNumCols; iCol1++)
      iPegAccounted[iCol1] = iSolAccounted[iCol1] = FALSE;
    
    /*
     *  Check for black & white hint pegs
     */
    for (iCol1 = 0; iCol1 < iNumCols; iCol1++)
      for (iCol2 = 0; iCol2 < iNumCols; iCol2++)
        if (iCol1 == iCol2)
        {
          if ((iPegColor[iCol1] == iSolution[iCol1]) &&
              (!iPegAccounted[iCol1]) && (!iSolAccounted[iCol1]))
          {
            iBlacks++;
            iPegAccounted[iCol1] = TRUE;
            iSolAccounted[iCol2] = TRUE;
          }
        }
        else
        {
          if ((iPegColor[iCol1] == iSolution[iCol2]) &&
              (iPegColor[iCol2] != iSolution[iCol2]) &&
              (iPegColor[iCol1] != iSolution[iCol1]) &&
              (!iPegAccounted[iCol1]) && (!iSolAccounted[iCol2]))
          {
            iWhites++;
            iPegAccounted[iCol1] = TRUE;
            iSolAccounted[iCol2] = TRUE;
          }
        }
    
    /*
     *  Draw the hint pegs
     */
    for (iCol1 = 0; iCol1 < iBlacks; iCol1++)
      gtk_led_set_color(GTK_LED(wHint[iNumRows -*iCurrentRow][iCol1]),
                        0, 0, 0);
    for (iCol2 = 0; iCol2 < iWhites; iCol2++)
      gtk_led_set_color(GTK_LED(wHint[iNumRows -*iCurrentRow][iCol1 +iCol2]),
                        65535, 65535, 65535);
    
    /*
     *  Compute the new score
     */
    iNewScore = (iNumColors*2 +iNumCols)*(iBlacks*3 +iWhites)*10 -uiGameTime
                -*iCurrentRow*100;
    if (iBlacks == iNumCols)
      iNewScore += (ROWS -*iCurrentRow +ROWS -iNumRows)*500;
    if (iNewScore < 0)
      iNewScore = 0;
      
    /*
     *  Make sure there have been some changes so we don't count the score twice
     */
    if (*iCurrentRow > 1)
    {
      iChanges = 0;
      for (iCol1 = 0; iCol1 < iNumCols; iCol1++)
      {
        if (iOldPegColor[iCol1] != iPegColor[iCol1])
          iChanges += 1;
        iOldPegColor[iCol1] = iPegColor[iCol1];
      }
      if (iChanges == 0)
        iNewScore = 0;
    }
    else
      for (iCol1 = 0; iCol1 < iNumCols; iCol1++)
        iOldPegColor[iCol1] = iPegColor[iCol1];
    
    /*
     *  Display the updated score
     */
    iScore += iNewScore;
    sprintf(sScore, "%d", iScore);
    gtk_label_set(GTK_LABEL(wScore), sScore);
    
  }
  
  /* Win game, lose game, or next row
   *
   */
  if (iBlacks == iNumCols)
  {
    gtk_timeout_remove(GameTimer);
    WinGameDialog();
  }
  else if (++*iCurrentRow <= iNumRows)
    EnableRow(*iCurrentRow);
  else
  {
    gtk_timeout_remove(GameTimer);
    LoseGameDialog();
  }
}


void ExitGame(GtkWidget *wWidget, GdkEvent *gdkEvent, gpointer data)
{
  SaveHighScores();
  SaveGameOptions();
  gtk_main_quit();
}
   

int main(int argc, char *argv[])
{
  GtkWidget	*wMainMenu;				/* main menu					*/
	
  /*
   *  Initialize the application 
   */
  gtk_init(&argc, &argv);
  ButtonSignal = 0;
  bAllowRepeatColors = TRUE;
  
  /*
   *  Seed the random number generator with the time
   */
  srand(time(NULL) +getpid());
  InitColors();
  
  /*
   *  Default game parameters
   */
  iNumRows = 12;
  iNumCols = 5;
  iNumColors = 7;
  LoadGameOptions();

  /* 
   *  Create the main window 
   */
  wMainWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(wMainWindow), SOFTWARE_VERSION);
  gtk_widget_set_usize(wMainWindow, 324, 256);
  gtk_signal_connect(GTK_OBJECT(wMainWindow), "destroy",
                     GTK_SIGNAL_FUNC(ExitGame), NULL);
  gtk_signal_connect(GTK_OBJECT(wMainWindow), "delete_event",
                     GTK_SIGNAL_FUNC(ExitGame), NULL);

  wBox1 = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(wMainWindow), wBox1);
  gtk_widget_show(wBox1);
  
  /*
   *  Create the main menu
   */
  wMainMenu = CreateMainMenu();
  gtk_box_pack_start(GTK_BOX(wBox1), wMainMenu, FALSE, FALSE, 2);
  gtk_widget_show(wMainMenu);
  
  /*
   *  Load the high scores file
   */
  LoadHighScores();
	
  /*
   *  Setup a new game
   */
  wGameBoard = InitGame(iNumRows, iNumCols, &iNumColors);
  gtk_box_pack_end(GTK_BOX(wBox1), wGameBoard, TRUE, TRUE, 2);
  gtk_widget_show(wGameBoard);

  /*
   *  Display the main window
   */
  gtk_widget_show(wMainWindow);
  
  /*
   *  Run the application
   */
  gtk_main();
	
  return 0;
}
